# -*- coding: utf-8 -*-
from __future__ import print_function
from os3.utils.console import pprint_list


def name_id_parent_function(elem):
    return elem[0], elem[1], elem[2]


def init_tree(process, name_id_parent_fn=None):
    name_id_parent_fn = name_id_parent_fn or name_id_parent_function
    from treelib import Tree
    tree = Tree()
    for children in process:
        tree.create_node(*name_id_parent_fn(children))
    return tree


class GradaleComponent(object):
    name = ''
    __clone_params__ = []

    def check_filters(self, **kwargs):
        for name, value in kwargs.items():
            if self.value(name) != value:
                return False
        return True

    def values(self, *interfaces, **kwargs):
        return {key: getattr(self, key) for key in interfaces}

    def value(self, interface):
        value = getattr(self, interface)
        if hasattr(value, '__call__'):
            value = value()
        return value

    def clone(self):
        params = {key: getattr(self, key) for key in self.__clone_params__}
        new_instance = self.__class__(**params)
        return new_instance

    def print_format(self):
        return self.name.encode('utf8').decode('utf-8', 'replace')

    def print(self):
        print(self.print_format())


class GradaleList(GradaleComponent):
    _tuple_filters = None
    _dict_filters = None
    _sort = None # []
    _iter = None

    def _add_filters(self, filters):
        self._dict_filters = self._dict_filters or {}
        self._dict_filters.update(filters)

    def _set_sort(self, *interfaces):
        self._sort = interfaces

    def filter(self, **kwargs):
        instance = self.clone()
        instance._add_filters(kwargs)
        return instance

    def sort(self, *interfaces):
        instance = self.clone()
        instance._set_sort(*interfaces)
        return instance

    def group_by(self, interface):
        pass

    def _get_iter(self):
        raise NotImplementedError

    def _prepare_iter(self):
        it = self._get_iter()
        if self._sort:
            it = map(self._prepare_next, it)
            # TODO: eliminar luego
            it = [x for x in it]
            # TODO: depurando. Los que son un archivo devuelven el método sin llamar. Directorios vacíos...
            # ¡¿una lista vacía?! Resultado esperado: int.
            it = sorted(it, key=lambda x: [x.value(interface) for interface in self._sort])
        return it

    def _next(self, reset=False):
        """Obtener el siguiente elemento de la iteración sin filtros
        """
        if not self._iter and not reset:
            self._iter = self._prepare_iter()
        return next(self._iter)

    def _prepare_next(self, elem):
        """Devolver el siguiente elemento de la iteración preparado
        """
        raise NotImplementedError

    def __prepare_next(self, elem):
        """Ejecutar _prepare_next solo si no es un GradaleComponent.
        """
        if isinstance(elem, GradaleComponent):
            return elem
        return self._prepare_next(elem)

    def list(self):
        return filter(self._elem_is_valid, [self.__prepare_next(elem) for elem in self._prepare_iter()])

    def _elem_is_valid(self, elem):
        """Comprobar si el elemento se puede devolver por los filtros
        """
        return elem.check_filters(**self._dict_filters or {})

    # def __next__(self):
    #     elem = None
    #     while True:
    #         elem = self.__prepare_next(self._next())
    #         if self._elem_is_valid(elem):
    #             break
    #     return elem
    #
    # def __iter__(self):
    #     # Probar a cambiar __next__ por __iter__ y el return del primero por yield
    #     # http://stackoverflow.com/questions/2776829/difference-between-pythons-generators-and-iterators
    # http://stackoverflow.com/questions/231767/what-does-the-yield-keyword-do-in-python
    #     return self

    # TODO: Pruebas. Código de arriba funcional
    def __iter__(self):
        while True:
            try:
                elem = self.__prepare_next(self._next())
            except StopIteration:
                self._iter = None
                raise StopIteration
            if not self._elem_is_valid(elem):
                continue
            yield elem

    # def next(self):
    #     """Retrocompatibilidad con Python2
    #     """
    #     return self.__next__()

    def print_format(self):
        return self.list_format()

    def tree_format(self, roots=None, fn_tree=None):
        roots = roots if roots is not None else self
        fn_tree = fn_tree or init_tree
        forest = [fn_tree(x) for x in roots]
        output = ''
        for tree in forest:
            output += str(tree)
        return output

    def list_format(self):
        return pprint_list([x.print_format() for x in self])

    def values(self, *interfaces, **kwargs):
        return super(GradaleList, self).values(*interfaces)

    def values_list(self, *interfaces, **kwargs):
        return [n.values(*interfaces, this=True) for n in self.list()]

    def value_list(self, interface, **kwargs):
        return [n.value(interface) for n in self.list()]


class StartsWithEqual(object):
    def __init__(self, name):
        self.name = name

    def __eq__(self, other):
        return self.name.startswith(other)

    def __ne__(self, other):
        return not self.__eq__(other)


# -*- coding: utf-8 -*-
import os
import unittest
from operator import itemgetter

from os3.fs.directory import Dir
from os3.tests.base import MockTreeNode


class TestList(MockTreeNode):

    def test_list(self):
        """Comprobar que todos los archivos y directorios se estén listando correctamente en el directorio padre
        """
        self.assertEqual(set([node.name for node in Dir(self.directory).ls()]), set(self.list_dir()))

    def test_tree(self):
        Dir(self.directory, deep=True).ls().tree_format()

    def test_subfilter(self):
        """Comprobar que los filtros sin deep estén funcionando
        """
        self.assertEqual(set([node.path for node in Dir(self.directory).ls().filter(type='d')]),
                         set(filter(os.path.isdir, self.list_dir(full_path=True))))

    def test_values(self):
        """Comprobar el funcionamiento de values en listas
        """
        self.assertEqual(sorted(Dir(self.directory).ls().filter(type='f').values_list('name', 'path', 'size'),
                                key=itemgetter('path')),
                         sorted([{'name': os.path.split(f)[1], 'size': os.path.getsize(f), 'path': f}
                              for f in filter(os.path.isfile, self.list_dir(full_path=True))], key=itemgetter('path')))

    def test_value(self):
        """Comprobar el funcionamiento de value en listas.
        """
        self.assertEqual(set(Dir(self.directory).ls().value_list('path')), set(self.list_dir(full_path=True)))

    def test_sort(self):
        """Comprobar que el sort funcione.
        """
        self.assertEqual(Dir(self.directory).ls().sort('size').values_list('path', 'size'),
                         sorted([{'path': path, 'size': os.path.getsize(path)}

                                 for path in self.list_dir(full_path=True)], key=itemgetter('size')))


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

def add_decorator(*targets, **kwargs):
    decorator = kwargs.pop('decorator', None)
    if decorator is None:
        raise ValueError('The decorator must be defined')
    args = tuple(map(decorator, targets))
    if len(args) == 1:
        return args[0]
    return args



# -*- coding: utf-8 -*-
from future.builtins import open
import os
import sys

import six

from os3.fs.entry import Entry


class File(Entry):
    __interfaces__ = ['name']
    __clone_params__ = ['path']
    _type = 'file'
    _open = None
    _mode = None

    def _get_open(self, mode='r', buffering=-1, encoding=None, errors=None, newline=None, closefd=True, opener=None):
        if self._open is None or self._mode != mode:
            self._mode = mode
            return self.open(mode, buffering, encoding, errors, newline, closefd, opener)
        else:
            return self._open

    def open(self, mode='r', buffering=-1, encoding=None, errors=None, newline=None, closefd=True, opener=None):
        if sys.version_info >= (3, 0):
            self._open = open(self.path, mode, buffering, encoding, errors, newline, closefd, opener)
        else:
            self._open = open(self.path, mode)
        return self

    def read(self, n=None, mode=None, buffering=-1, encoding=None, errors=None, newline=None, closefd=True,
             opener=None):
        mode = mode or self._mode or 'r'
        if n is not None and not isinstance(n, int):
            mode = n
            n = None
        if six.PY2 and n is None:
            # PATCH: Python2 requiere que n sea un int siempre
            n = -1
        data = self._get_open(mode, buffering, encoding, errors, newline, closefd, opener).read(n)
        if n is None or n is -1:
            self.seek(0)
        return data

    def write(self, data, mode=None, buffering=-1, encoding=None, errors=None, newline=None, closefd=True, opener=None):
        mode = mode or self._mode or 'w'
        if six.PY2 and 'b' not in mode and isinstance(data, bytes):
            data = data.decode('utf-8')
        self._get_open(mode, buffering, encoding, errors, newline, closefd, opener).write(data)

    def touch(self):
        if not self.lexists():
            self.write('')
        return self

    def tell(self):
        return self._open.tell()

    @property
    def size(self):
        return os.path.getsize(self.path)

    def readlines(self, n=None, mode='r', buffering=-1, encoding=None, errors=None, newline=None, closefd=True,
                  opener=None, breaklines=True):
        if n is not None and not isinstance(n, int):
            mode = n
            n = 0
        if six.PY2 and n is None:
            # PATCH: Python2 requiere que n sea un int siempre
            n = 0
        lines = self._get_open(mode, buffering, encoding, errors, newline, closefd, opener).readlines(n)

        def remove_breakline(line):
            if line.endswith('\r\n'):
                return line[:-2]
            if line.endswith('\n'):
                return line[:-1]
            return line
        if not breaklines:
            lines = list(map(remove_breakline, lines))
        return lines

    def readline(self, mode='r', buffering=-1, encoding=None, errors=None, newline=None, closefd=True, opener=None,
                 breakline=True):
        line = self.readlines(1, mode, buffering, encoding, errors, newline, closefd, opener, breakline)
        if not line:
            return None
        return line[0]

    def seek(self, i):
        return self._open.seek(i)

    def __repr__(self):
        return self.name


# -*- coding: utf-8 -*-
import os
import shutil

import six

from os3.components import GradaleComponent, StartsWithEqual


class Entry(GradaleComponent):
    _type = None
    path = ''
    root = None

    def __new__(cls, *args, **kwargs):
        from os3.fs.directory import Dir
        from os3.fs.file import File

        if cls != Entry or not args:
            return GradaleComponent.__new__(cls)
        if args and isinstance(args[0], Entry):
            path = args[0].path
        else:
            path = os.path.realpath(os.path.expanduser(args[0])) if args else None
        if path and os.path.isdir(path):
            return Dir.__new__(Dir, *args, **kwargs)
        elif path and os.path.isfile(path):
            return File.__new__(File, *args, **kwargs)
        return File.__new__(File, *args, **kwargs)
        # return GradaleComponent.__new__(cls)

    def __init__(self, path, **kwargs):
        self.path = self._get_path(path)

    @property
    def name(self):
        return os.path.split(self.path)[1]

    @property
    def size(self):
        return os.path.getsize(self.path)

    @property
    def ctime(self):
        return os.path.getctime(self.path)

    @property
    def mtime(self):
        return os.path.getmtime(self.path)

    @property
    def atime(self):
        return os.path.getatime(self.path)

    @classmethod
    def get_cls(cls, path):
        from os3.fs.directory import Dir
        from os3.fs.file import File

        if os.path.isdir(path):
            # return File
            return Dir
        else:
            return File

    @classmethod
    def get_node(cls, path):
        path = cls._get_path(path)
        return cls.get_cls(path)(path)

    @property
    def type(self):
        return StartsWithEqual(self._type)

    def exists(self):
        return os.path.exists(self.path)

    def lexists(self):
        return os.path.lexists(self.path)

    def is_dir(self):
        return self.type == 'directory'

    def is_file(self):
        return self.type == 'file'

    def bak(self):
        if not self.lexists():
            return self
        i = -1
        bak_name = self.path + '.bak'
        while True:
            new_bak_name = bak_name
            if i > -1:
                new_bak_name += str(i)
            if not os.path.lexists(new_bak_name):
                shutil.move(self.path, new_bak_name)
                break
            i += 1
        return self

    def symlink(self, link_name):
        os.symlink(self.path, get_path(link_name))

    def copy(self, dst, symlinks=False, ignore=None):
        shutil.copytree(self.path, os.path.expanduser(dst), symlinks, ignore)

    def parent(self):
        from os3.fs.directory import Dir
        path = os.path.split(self.path)[0]
        if self.root and not path.startswith(self.root):
            return None
        return Dir(path)

    def sub(self, subpath):
        # TODO: Esto no debería estar limitado a los directorios?
        return Entry(os.path.join(self.path, get_path(subpath)))

    def depth(self):
        # TODO: tener en cuenta self.root
        return self.path.split(os.sep)

    @classmethod
    def _get_path(cls, path):
        """Obtener el path de una variable path, que puede ser el propio path, o un DirEntry
        """
        if not isinstance(path, (str, six.string_types)):
            # Es un DirEntry
            path = path.path
        return os.path.abspath(os.path.expanduser(path))


def get_path(node):
    if isinstance(node, Entry):
        return node.path
    return os.path.expanduser(node)



# -*- coding: utf-8 -*-
import os
import sys

from os3.fs import __init__, Entry, get_path


def bak_target_decorator(fn):
    def decorator(src, dst=None, **kwargs):
        src, dst = Entry(src), Entry(dst) if dst else dst
        if dst and os.path.islink(dst.path) and os.path.realpath(dst.path) == os.path.realpath(src.path):
            return
        if not dst and src.exists():
            return
        elif dst and dst.lexists():
            dst.bak()
        return fn(src, dst, **kwargs) if dst is not None else fn(src, **kwargs)
    return decorator


def get_abspath(node):
    if isinstance(node, Entry):
        return node.path
    return os.path.abspath(os.path.expanduser(node))


def get_node(path):
    if not isinstance(path, Entry):
        return Entry(path)
    return path


def symlink(source, link_name):
    return get_node(source).symlink(link_name)


def mkdir(path, mode=511, exists_ok=False):
    if sys.version_info >= (3,0):
        return os.makedirs(get_path(path), mode, exists_ok)
    return os.makedirs(get_path(path), mode)


def cp(src, dst, symlinks=False, ignore=None):
    return get_node(src).copy(dst, symlinks, ignore)


# -*- coding: utf-8 -*-
import os
import tempfile
import unittest
from unittest import TestCase

from os3.fs.entry import Entry


class TestEntry(TestCase):
    def test_bak(self):
        name = tempfile.NamedTemporaryFile().name
        Entry(name).touch().bak()
        self.assertFalse(os.path.lexists(name))
        bak_name = '{}.bak'.format(name)
        self.assertTrue(os.path.lexists(bak_name))
        Entry(name).touch().bak()
        bak_name2 = '{}.bak0'.format(name)
        self.assertTrue(os.path.lexists(bak_name2))
        os.remove(bak_name)
        os.remove(bak_name2)

    def test_sub(self):
        self.assertEqual(Entry('/foo').sub('bar').path, '/foo/bar')


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import os
import tempfile
from unittest import TestCase

import shutil


class MockTreeNode(TestCase):
    tree = [
        'dir01', 'dir02', 'dir03',
        'dir01/subdir01', 'dir02/subdir01', 'dir02/subdir02', 'dir02/subdir03',
        'dir01/subdir01/subsubdir01', 'dir01/subdir01/subsubdir02',
        'dir01/subdir01/subsubdir01/subsubsubdir01',
    ]
    files_by_dir = 6

    def setUp(self):
        self.directory = tempfile.mkdtemp()
        self.create_dirty_files()
        self.create_tree()

    def create_tree(self):
        for d in self.tree:
            self.create_dirty_files(d)

    def create_dirty_files(self, node=''):
        def prefix_lvl(name):
            if not node:
                return name
            return 'lvl%02d_%s' % (len(node.split('/')), name)
        def join_node(name, prefix=True):
            if prefix:
                name = prefix_lvl(name)
            if not node:
                return name
            return os.path.join(node, name)
        self.create_directory(node)
        self.create_file(join_node('empty_file.txt'))
        self.create_file(join_node('empty_file_without_ext'))
        self.create_file(join_node('.hidden_file.txt'))
        self.create_file(join_node('file_01.spam'), 'Spam ' * 256)
        self.create_file(join_node('file_02.eggs'), 'Eggs ' * 512)
        self.create_file(join_node('file_03.spam'), 'Eggs ' * 128)
        self.create_directory(join_node('empty_dir'))
        self.create_directory(join_node('.hidden_empty_dir', False))

    def create_directory(self, name):
        d = os.path.join(self.directory, name)
        if os.path.exists(d):
            return
        os.makedirs(d)

    def create_file(self, name, data=''):
        with open(os.path.join(self.directory, name), 'w') as f:
            f.write(data)

    def list_dir(self, path='', full_path=False):
        path = os.path.join(self.directory, path)
        if full_path:
            return [os.path.join(self.directory, name) for name in os.listdir(path)]
        return os.listdir(path)

    def deep_list_dir(self, path=''):
        path = os.path.join(self.directory, path)
        nodes = set()
        for dirpath, dirnames, filenames in os.walk(path):
            for node in dirnames + filenames:
                nodes.add(os.path.join(dirpath, node))
        return nodes

    def tearDown(self):
        shutil.rmtree(self.directory)



# -*- coding: utf-8 -*-
import subprocess

from os3.fs.entry import Entry

from os3.components import GradaleList, init_tree


def all_childrens(process):
    yield process
    for child in process._children:
        for subchild in all_childrens(child):
            yield subchild


def set_cache_tree(tree):
    tree = list(tree)
    process_by_pid = {process.pid:process for process in tree}
    for process in tree:
        process._parent = None
        process._children = []
    for process in tree:
        parent = process.parent()
        pid = parent.pid if parent else None
        process._parent = process_by_pid.get(pid)
        if not process._parent:
            continue
        process._parent._children.append(process)
    return tree


def name_id_parent_fn(process):
    parent = process._parent
    parent = parent.pid if parent is not None else None
    return process.name(), process.pid, parent


class Processes(GradaleList):

    def _get_iter(self):
        import psutil
        return iter(psutil.pids())

    def _prepare_next(self, process):
        from .process import Process
        return Process(process)

    def print_format(self):
        return self.tree_format()

    def tree_format(self, roots=None, fn_tree=None):
        roots = set_cache_tree(self)
        roots = filter(lambda x: not x._parent, roots)
        return super(Processes, self).tree_format(roots, lambda x: init_tree(all_childrens(x), name_id_parent_fn))

    def __repr__(self):
        return self.print_format()


class Run(object):
    def __init__(self, cmd, cwd=None):
        cmd = list(map(lambda x: x.path if isinstance(x, Entry) else x, cmd))
        if isinstance(cwd, Entry):
            cwd = cwd.path
        self.process = subprocess.Popen(cmd, cwd=cwd)
        self.process.wait()

    @property
    def returncode(self):
        return self.process.returncode



# -*- coding: utf-8 -*-
from itertools import combinations

import sys

import six


def get_terminal_size():
    # http://stackoverflow.com/questions/566746/how-to-get-console-window-width-in-python
    import os
    env = os.environ
    def ioctl_GWINSZ(fd):
        try:
            import fcntl, termios, struct, os
            cr = struct.unpack('hh', fcntl.ioctl(fd, termios.TIOCGWINSZ,
        '1234'))
        except:
            return
        return cr
    cr = ioctl_GWINSZ(0) or ioctl_GWINSZ(1) or ioctl_GWINSZ(2)
    if not cr:
        try:
            fd = os.open(os.ctermid(), os.O_RDONLY)
            cr = ioctl_GWINSZ(fd)
            os.close(fd)
        except:
            pass
    if not cr:
        cr = (env.get('LINES', 25), env.get('COLUMNS', 80))

        ### Use get(key[, default]) instead of a try/catch
        #try:
        #    cr = (env['LINES'], env['COLUMNS'])
        #except:
        #    cr = (25, 80)
    return int(cr[1]), int(cr[0])


def pprint_list(input_list):
    # http://stackoverflow.com/questions/25026556/output-list-like-ls
    (term_width, term_height) = get_terminal_size()
    if len( str(input_list) ) <= term_width:
        print(input_list)
        return

    repr_list = [x.decode('utf-8') if six.PY2 else x for x in input_list]
    # repr_list = [x.decode('utf-8') for x in input_list]
    # repr_list = [repr(x) for x in input_list]
    # min_chars_between = 3 # a comma and two spaces
    min_chars_between = 2
    # usable_term_width = term_width - 3 # For '[ ' and ']' at beginning and end
    usable_term_width = term_width
    min_element_width = min( len(x) for x in repr_list ) + min_chars_between
    max_element_width = max( len(x) for x in repr_list ) + min_chars_between
    if max_element_width >= usable_term_width:
        ncol = 1
        col_widths = [1]
    else:
        # Start with max possible number of columns and reduce until it fits
        ncol = int(min( len(repr_list), usable_term_width / min_element_width  ))
        while True:
            col_widths = [ max( len(x) + min_chars_between \
                                for j, x in enumerate( repr_list ) if j % ncol == i ) \
                                for i in range(ncol) ]
            if sum( col_widths ) <= usable_term_width: break
            else: ncol -= 1

    # # sys.stdout.write('[ ')
    # for i, x in enumerate(repr_list):
    #     # if i != len(repr_list)-1: x += ','
    #     sys.stdout.write( x.ljust( col_widths[ i % ncol ] ) )
    #     if i == len(repr_list) - 1:
    #         sys.stdout.write('\n')
    #         pass
    #     elif (i+1) % ncol == 0:
    #         sys.stdout.write('\n')
    #         # sys.stdout.write('\n  ')

    # sys.stdout.write('[ ')
    output = ''
    for i, x in enumerate(repr_list):
        # if i != len(repr_list)-1: x += ','
        output += x.ljust(col_widths[i % ncol])
        if (i + 1) % ncol == 0:
            output += '\n'
        # sys.stdout.write('\n  ')
    return output


# def print_columns(elements):
#     # http://stackoverflow.com/questions/9989334/create-nice-column-output-in-python
#     columns = list(grouper(2, elements))
#     col_width = max(len(word) if word else 0 for row in columns for word in row) + 2  # padding
#     return '\n'.join(["".join(word.ljust(col_width) if word else '' for word in row) for row in columns ])



# -*- coding: utf-8 -*-
import os
import sys
import stat

import six


class FakeDirEntry(object):
    _stat = None

    def __init__(self, path):
        self.path = path

    @property
    def name(self):
        return self.path.split(self.path)[1]

    def is_dir(self):
        return stat.S_ISDIR(self.stat().st_mode)

    def is_symlink(self):
        return stat.S_ISLNK(self.stat().st_mode)

    def is_file(self):
        return stat.S_ISREG(self.stat().st_mode)

    def stat(self):
        if not self._stat:
            self._stat = os.stat(self.path)
        return self._stat

    def inode(self):
        return self.stat().st_ino


def scandir(path='.'):
    if sys.version_info >= (3, 5):
        return os.scandir(path)
    try:
        import scandir
    except ImportError:
        return map(FakeDirEntry, os.listdir(path))
    else:
        return scandir(path)


def get_path(path):
    if not isinstance(path, (str, six.string_types)):
        path = path.path
    return path


def deep_scandir(path, deep=False, cls=None, filter=None, traverse_filter=None):
    filter = filter or (lambda x: True)
    traverse_filter = traverse_filter or (lambda x: True)

    for item in scandir(path):
        item = os.path.join(get_path(path), get_path(item))
        item = item if cls is None else cls(item)
        traverse_result = item.is_dir() and traverse_filter(item)
        if deep and traverse_result:
            new_deep = deep
            if not isinstance(deep, bool) and isinstance(deep, int):
                new_deep = deep - 1
            # yield from deep_scandir(item.path, new_deep)
            for subitem in deep_scandir(item.path, new_deep, cls):
                yield subitem
        if item.is_dir() and not traverse_result:
            # Si es un directorio y no cumple el traverse_result, no merece probar filter
            continue
        if filter(item):
            yield item



# -*- coding: utf-8 -*-
from future.builtins import open as uopen
import os
import tempfile
import unittest
from unittest import TestCase

import six

from os3.fs.directory import Dir
from os3.fs.file import File
from os3.tests.base import MockTreeNode


class TestDirectory(MockTreeNode):

    def test_deep(self):
        self.assertEqual(set(Dir(self.directory).ls(deep=True).value_list('path')), self.deep_list_dir())

    def test_not_repeated(self):
        items = Dir(self.directory).ls(deep=True).value_list('path')
        self.assertEqual(len(items), len(set(items)))

    def test_max_deep(self):
        self.assertEqual(set(Dir(self.directory).ls(deep=0).value_list('path')), set(self.list_dir(full_path=True)))
        self.assertNotIn(os.path.join(self.directory, 'dir01/subdir01/subsubdir01'),
                         Dir(self.directory).ls(deep=1).value_list('path'))
        self.assertIn(os.path.join(self.directory, 'dir01/subdir01/subsubdir01'),
                      Dir(self.directory).ls(deep=2).value_list('path'))

    def test_filters_deep(self):
        # print(len([x for x in Dir(self.directory).ls(deep=True).filter(type='f')]))
        files = list(Dir(self.directory).ls(deep=True).filter(type='f').value_list('path'))
        six.assertCountEqual(self, files, set(files))
        self.assertEqual(len(files), (len(self.tree) + 1) * self.files_by_dir)

    def test_pre_filters_deep(self):
        from os3.fs.file import File
        files = list(Dir(self.directory).ls(type='f', deep=True))
        self.assertEqual(len(files), self.files_by_dir)
        for file in files:
            self.assertIsInstance(file, File)

    def test_repr(self):
        self.assertEqual(repr(Dir(self.directory)), Dir(self.directory).name)

    def test_print_format(self):
        self.assertIn(Dir(self.directory).name, Dir(self.directory).print_format())


class TestFile(TestCase):
    def setUp(self):
        self.filename = tempfile.NamedTemporaryFile().name

    def test_read(self):
        data = 'FOO' * 10
        with open(self.filename, 'w') as f:
            f.write(data)
        self.assertEqual(File(self.filename).read(), data)

    def test_repeat_read(self):
        data = 'SPAM' * 10
        with open(self.filename, 'w') as f:
            f.write(data)
        f = File(self.filename)
        self.assertEqual(f.read(), data)
        self.assertEqual(f.read(), data)

    def test_readlines(self):
        lines = list(map(str, range(10)))
        with open(self.filename, 'w') as f:
            f.write('\n'.join(lines))
        self.assertEqual(File(self.filename).readlines(breaklines=False), lines)
        with open(self.filename, 'wb') as f:
            f.write(b'\r\n'.join(map(lambda x: x.encode('utf-8'), lines)))
        self.assertEqual(File(self.filename).readlines(breaklines=False), lines)

    def test_readline(self):
        lines = list(map(str, range(10)))
        with open(self.filename, 'w') as f:
            f.write('\n'.join(lines))
        f = File(self.filename)
        read_lines = []
        while True:
            line = f.readline(breakline=False)
            if line is None:
                break
            read_lines.append(line)
        self.assertEqual(read_lines, lines)

    def test_read_n_data(self):
        data = u'ññññ'
        with uopen(self.filename, 'w') as f:
            f.write(data)
        self.assertEqual(File(self.filename).read(4), data)

    def test_read_n_bytes_data(self):
        data = u'ññññ'
        with uopen(self.filename, 'wb') as f:
            f.write(data.encode('utf-8'))
        self.assertEqual(File(self.filename).read(4, 'rb'), data.encode('utf-8')[:4])

    def tearDown(self):
        os.remove(self.filename)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import os

from colorama import Fore, Style

from os3.components import GradaleList, init_tree
from os3.fs.entry import Entry
from os3.utils.nodes import deep_scandir


def name_id_parent_fn(item):
    parent = item.parent()
    parent = parent.path if parent is not None else None
    return item.name, item.path, parent


def init_dir_tree(directory, *args):
    directory = directory.clone()
    directory.root = directory.path
    directories = list(sorted(directory, key=lambda x: x.depth()))
    return init_tree([directory] + directories, name_id_parent_fn)


class Dir(Entry):
    _type = 'directory'

    def ls(self, deep=None, **kwargs):
        return DirList(self.path, deep, **kwargs)

    def print_format(self):
        return '{Fore.BLUE}{name}{Style.RESET_ALL}'.format(name=self.name, Fore=Fore, Style=Style)

    def __repr__(self):
        return self.name


class DirList(Dir, GradaleList):
    _pre_filters = None
    __interfaces__ = ['name']
    __clone_params__ = ['path', 'deep']
    _ls = None

    def __init__(self, path=None, deep=None, **kwargs):
        # TODO: renombrar deep a depth
        path = path or os.getcwd()
        super(Dir, self).__init__(path)
        self.deep = deep
        self.root = kwargs.pop('root', None)
        self._pre_filters = kwargs

    def _get_iter(self):
        return deep_scandir(self.path, self.deep, cls=Entry, filter=self._filter, traverse_filter=self._traverse_filter)
        # return iter(os.listdir(self.path))

    def _prepare_next(self, elem):
        return Entry.get_node(elem.path)
        # return Node.get_node(os.path.join(self.path, elem))

    def _filter(self, elem):
        return elem.check_filters(**self._pre_filters or {}) and elem.check_filters(**self._dict_filters or {})

    def _traverse_filter(self, elem):
        return elem.check_filters(**self._pre_filters or {})

    def tree_format(self, roots=None, fn_tree=None, roots_filter_fn=None):
        return super(Dir, self).tree_format([self], init_dir_tree)

    def print_format(self):
        return GradaleList.print_format(self)

    def __repr__(self):
        return self.print_format()